#
# Copyright (c) 2008-2018 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

__docformat__ = 'restructuredtext'

from zope.interface import Interface

from pyams_content.component.paragraph.interfaces import IParagraphContainer
from pyams_content.component.paragraph.portlet.interfaces import IParagraphContainerPortletSettings, \
    IParagraphNavigationPortletSettings
from pyams_content.features.renderer.interfaces import ISharedContentRenderer
from pyams_content.shared.site.interfaces import ISiteContainer
from pyams_default_theme.interfaces import IContentSummaryInfo
from pyams_portal.interfaces import IPortalContext, IPortletRenderer
from pyams_portal.portlet import PortletRenderer
from pyams_sequence.interfaces import IInternalReference, ISequentialIdInfo
from pyams_skin.layer import IPyAMSLayer
from pyams_template.template import template_config
from pyams_utils.adapter import adapter_config

from pyams_default_theme import _


@adapter_config(context=(IPortalContext, IPyAMSLayer, Interface, IParagraphContainerPortletSettings),
                provides=IPortletRenderer)
@template_config(template='templates/content.pt', layer=IPyAMSLayer)
class ParagraphContainerPortletRenderer(PortletRenderer):
    """Paragraph container default portlet renderer"""

    label = _("Default paragraphs renderer")

    renderers = ()

    def update(self):
        super(ParagraphContainerPortletRenderer, self).update()
        settings = self.settings
        registry = self.request.registry
        container = IParagraphContainer(self.context, None)
        if container is not None:
            paragraphs = container.get_visible_paragraphs(settings.paragraphs, settings.anchors_only,
                                                          settings.factories)
            renderers = [paragraph.get_renderer(self.request) for paragraph in paragraphs]
        else:
            renderers = [adapter for name, adapter in sorted(registry.getAdapters((self.context, self.request),
                                                                                  ISharedContentRenderer),
                                                             key=lambda x: x[1].weight)]
        self.renderers = list(filter(lambda x: x is not None, renderers))
        [renderer.update() for renderer in self.renderers]

    def get_navigation_links(self):

        def test_item(nav_item):
            item_sequence = ISequentialIdInfo(nav_item, None)
            if (item_sequence is not None) and (item_sequence.oid == context_sequence.oid):
                return True
            if IInternalReference.providedBy(nav_item) and (nav_item.reference == context_sequence.hex_oid):
                return True

        prev_nav, next_nav = None, None
        context_sequence = ISequentialIdInfo(self.context, None)
        if context_sequence is not None:
            display_context = self.request.display_context
            if ISiteContainer.providedBy(display_context):
                registry = self.request.registry
                previous_item = None
                items = display_context.get_visible_items(self.request)
                for item in items:
                    if test_item(item):
                        prev_nav = registry.queryMultiAdapter((previous_item, self.request), IContentSummaryInfo)
                        break
                    previous_item = item
                try:
                    next_item = next(items)
                except StopIteration:
                    pass
                else:
                    next_nav = registry.queryMultiAdapter((next_item, self.request), IContentSummaryInfo)
        return prev_nav, next_nav


@adapter_config(context=(IPortalContext, IPyAMSLayer, Interface, IParagraphNavigationPortletSettings),
                provides=IPortletRenderer)
@template_config(template='templates/navigation.pt', layer=IPyAMSLayer)
class ParagraphNavigationPortletRenderer(PortletRenderer):
    """Paragraphs navigation default portlet renderer"""

    label = _("Default paragraphs navigation")

    paragraphs = ()

    def update(self):
        super(ParagraphNavigationPortletRenderer, self).update()
        settings = self.settings
        container = IParagraphContainer(self.context, None)
        if container is not None:
            self.paragraphs = container.get_visible_paragraphs(settings.paragraphs, settings.anchors_only,
                                                               settings.factories)
